/* ========================================================================
 * Copyright (c) 2009 The University of Washington
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================================
 */

package edu.internet2.middleware.subject.provider;

import java.util.Set;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.internet2.middleware.subject.Source;
import edu.internet2.middleware.subject.Subject;
import edu.internet2.middleware.subject.SubjectType;



/**
 * Ldap Subject implementation.
 */
public class LdapSubject 
	implements Subject {
	

	private static Log log = LogFactory.getLog(LdapSubject.class);
	
	private LdapSourceAdapter adapter;
	
	private String id;
	private String name;
        private String description = null;
	private SubjectType type = null;
	private Map attributes = null;
	
	/*
	 * Constructor called by SourceManager.
	 */
	protected LdapSubject(String id, String name, String description,
			SubjectType type, LdapSourceAdapter adapter) {
		log.debug("Name = "  + name);
		this.id = id;
		this.name = name;
		this.type = type;
                this.description = description;
		this.adapter = adapter;
	}

	
	/**
	 * {@inheritDoc}
	 */
	public String getId() {
		return this.id;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public SubjectType getType() {
		return this.type;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getName() {
		return this.name;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getDescription() {
		return this.description;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getAttributeValue(String name) {
		if (attributes == null) {
			this.adapter.loadAttributes(this);
		}
		Set values = (Set)this.attributes.get(name);
		if (values != null) {
			return ((String[])values.toArray(new String[0]))[0];
		}
		else {
			return null;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public Set getAttributeValues(String name) {
		if (attributes == null) {
			this.adapter.loadAttributes(this);
		}
		return (Set)this.attributes.get(name);
	}

	/**
	 * {@inheritDoc}
	 */
	public Map getAttributes() {
		if (attributes == null) {
			this.adapter.loadAttributes(this);
		}
		return attributes;
	}

	/**
	 * {@inheritDoc}
	 */
	public Source getSource() {
		return this.adapter;
	}
	
	protected void setAttributes(Map attributes) {
		this.attributes = attributes;
	}
}
